using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Tools;

namespace POC
{
    internal partial class HideShowColumns : Form
    {
        public HideShowColumns()
        {
            InitializeComponent();
        }

        private List<ColumnInfo> _addedItems = new List<ColumnInfo>();
        internal List<ColumnInfo> AddedItems
        {
            get
            {
                return _addedItems;
            }
        }

        private List<ColumnInfo> _removedItems = new List<ColumnInfo>();
        internal List<ColumnInfo> RemovedItems
        {
            get
            {
                return _removedItems;
            }
        }

        Dictionary<ColumnInfo, bool> _items = new Dictionary<ColumnInfo, bool>();
        internal Dictionary<ColumnInfo, bool> Items
        {
            set
            {
                _items = value;
                TreeNodeAdv rootNode = uxColumns.Nodes[0];
                rootNode.Nodes.Clear();
                TreeNodeAdv [] columnInfoNodes = new TreeNodeAdv[_items.Count];
                int i = 0;
                foreach (ColumnInfo column in _items.Keys)
                {
                    TreeNodeAdv columnInfoNode = new TreeNodeAdv(column.ToString());
                    columnInfoNode.TagObject = column;
                    columnInfoNode.CheckState = _items[column] ? CheckState.Checked : CheckState.Unchecked;
                    columnInfoNodes[i++] = columnInfoNode;
                }
                rootNode.Nodes.AddRange(columnInfoNodes);
                rootNode.ExpandAll();
            }
        }

        private void OnLoad(object sender, EventArgs e)
        {
            uxColumns.AfterCheck += new Syncfusion.Windows.Forms.Tools.TreeNodeAdvEventHandler(OnAfterCheck);
            uxColumns.BeforeCheck += new TreeViewAdvBeforeCheckEventHandler(OnBeforeCheck);
        }

        private void OnBeforeCheck(object sender, TreeNodeAdvBeforeCheckEventArgs e)
        {
            if (e.NewCheckState != CheckState.Unchecked)
                return;

            if (e.Node.Level == 1)
                e.Cancel = true;

            if (uxColumns.CheckedNodes.Count - 1 <= 0) // - 2 - for the newly unchecked node
                e.Cancel = true;
        }

        private void OnAfterCheck(object sender, Syncfusion.Windows.Forms.Tools.TreeNodeAdvEventArgs e)
        {
            if (e.Node.Level == 1)
                return;

            if (e.Node.CheckState == CheckState.Checked)
            {
                ColumnInfo column = (ColumnInfo)e.Node.TagObject;
                _removedItems.Remove(column);
                if (!_items[column])
                    _addedItems.Add(column);
            }
            else if (e.Node.CheckState == CheckState.Unchecked)
            {
                ColumnInfo column = (ColumnInfo)e.Node.TagObject;
                _addedItems.Remove(column);
                if (_items[column])
                    _removedItems.Add(column);
            }
        }
    }
}