using System;
using System.Collections.Generic;
using System.Text;

using Syncfusion.Windows.Forms.Grid;
using System.Data;

namespace POC
{
    public class TableDataProvider : IVirtualGridDataProvider
    {
        public TableDataProvider() {}
        
        private DataTable _dataSource = null;
        public DataTable DataSource
        {
            get
            {
                return _dataSource;
            }
            set
            {
                _dataSource = value;
                if(ResetVolatileData != null)
                    this.ResetVolatileData(this, EventArgs.Empty);
            }
        }

        private bool _appendAtFront = false;
        public bool AppendAtFront
        {
            get { return _appendAtFront; }
            set { _appendAtFront = value; }
        }

        private string _format = string.Empty;
        public string DateDisplayFormat
        {
            get
            {
                return _format;
            }
            set
            {
                _format = value;
            }
        }

        public void AppendRows(DataRow[] rows)
        {
            if (_dataSource == null)
                throw new ArgumentException("Set DataSource before appending rows.");

            _dataSource.BeginLoadData();
            foreach (DataRow row in rows)
            {
                //if(_appendAtFront)
                //    _dataSource.Rows.InsertAt(row, 0);
                //else
                    _dataSource.Rows.Add(row);
            }
            _dataSource.EndLoadData();

            if (ResetVolatileData != null)
                this.ResetVolatileData(this, EventArgs.Empty);
        }

        public void Refresh()
        {
            if (ResetVolatileData != null)
                ResetVolatileData(this, EventArgs.Empty);
        }

        #region IVirtualGridDataProvider Members

        public virtual void QueryRowCount(GridRowColCountEventArgs e)
        {
            e.Count = _dataSource != null ? _dataSource.DefaultView.Count : 0;
            e.Handled = true;
        }

        public virtual void QueryColCount(GridRowColCountEventArgs e)
        {
            e.Count = _dataSource != null ? _dataSource.Columns.Count : 0;
            e.Handled = true;
        }

        public virtual void QueryCellInfo(GridQueryCellInfoEventArgs e)
        {
            if (_dataSource != null)
            {
                if (e.RowIndex == 0)
                {
                    if (e.ColIndex > 0 && e.ColIndex <= _dataSource.Columns.Count)
                    {
                        e.Style.CellValue = _dataSource.Columns[e.ColIndex - 1].Caption;
                        e.Style.CellValueType = _dataSource.Columns[e.ColIndex - 1].DataType;
                        e.Handled = true;
                    }
                }
                if (e.ColIndex > 0 && e.RowIndex > 0 && e.ColIndex <= _dataSource.Columns.Count && e.RowIndex <= _dataSource.Rows.Count)
                {
                    if (e.Style.CellValueType == typeof(bool))
                        e.Style.CellType = "CheckBox";
                    if(_appendAtFront)
                        e.Style.CellValue = _dataSource.DefaultView[_dataSource.Rows.Count - e.RowIndex].Row[e.ColIndex - 1];
                    else
                        e.Style.CellValue = _dataSource.DefaultView[e.RowIndex - 1].Row[e.ColIndex - 1];

                    //if (_dataSource.Columns[e.ColIndex - 1].DataType == typeof(DateTime))
                    //    e.Style.Format = _format;
                    e.Handled = true;
                }
            }
        }

        public virtual void QueryRowHeight(GridRowColSizeEventArgs e) { }

        public virtual void QueryColWidth(GridRowColSizeEventArgs e) { }

        public virtual void QueryCoveredRange(GridQueryCoveredRangeEventArgs e) { }

        public event EventHandler ResetVolatileData;

        #endregion
    }
}
